@@ -5,6 +5,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/sysmacros.h>
 
 #include "cr_options.h"
 #include "imgset.h"
@@ -12,12 +13,11 @@
 #include "fs-magic.h"
 #include "kerndat.h"
 #include "image.h"
+#include "util.h"
 #include "mount.h"
 #include "proc_parse.h"
 #include "servicefd.h"
 #include "file-lock.h"
-#include "parasite.h"
-#include "parasite-syscall.h"
 
 struct file_lock_rst {
 	FileLockEntry *fle;
@@ -339,39 +339,16 @@ static int restore_file_locks(int pid)
 	return ret;
 }
 
-static int restore_file_locks_legacy(int pid)
-{
-	int ret = -1;
-	struct cr_img *img;
-	FileLockEntry *fle;
-
-	img = open_image(CR_FD_FILE_LOCKS_PID, O_RSTR, pid);
-	if (!img)
-		return -1;
-
-	while (1) {
-		ret = pb_read_one_eof(img, &fle, PB_FILE_LOCK);
-		if (ret <= 0)
-			break;
-
-		ret = restore_file_lock(fle);
-		file_lock_entry__free_unpacked(fle, NULL);
-		if (ret)
-			break;
-	}
-
-	close_image(img);
-	return ret;
-}
-
 int prepare_file_locks(int pid)
 {
 	if (!opts.handle_file_locks)
 		return 0;
 
-	pr_info("Restore file locks.\n");
-	if (file_locks_cinfo.flags & COLLECT_HAPPENED)
-		return restore_file_locks(pid);
+	if (!(file_locks_cinfo.flags & COLLECT_HAPPENED)) {
+		pr_warn("Per-pid file locks are deprecated\n");
+		return -1;
+	}
+
+	return restore_file_locks(pid);
 
-	return restore_file_locks_legacy(pid);
 }
