@@ -5,8 +5,7 @@
 
 #include "sockets.h"
 #include "files.h"
-#include "list.h"
-#include "protobuf.h"
+#include "common/list.h"
 #include "images/sk-inet.pb-c.h"
 
 #define INET_ADDR_LEN		48 /* max of INET_ADDRSTRLEN and INET6_ADDRSTRLEN */
@@ -72,9 +71,9 @@ extern int dump_one_tcp(int sk, struct inet_sk_desc *sd);
 extern int restore_one_tcp(int sk, struct inet_sk_info *si);
 
 #define SK_EST_PARAM	"tcp-established"
+#define SK_INFLIGHT_PARAM "skip-in-flight"
 
 extern int check_tcp(void);
-extern mutex_t *inet_get_reuseaddr_lock(struct inet_sk_info *ii);
 
 struct task_restore_args;
 int prepare_tcp_socks(struct task_restore_args *);
