@@ -9,13 +9,12 @@
 #include "pstree.h"
 #include "util.h"
 #include "cr_options.h"
+#include "lsm.h"
 
 #include "protobuf.h"
 #include "images/inventory.pb-c.h"
 #include "images/creds.pb-c.h"
 
-#undef CONFIG_HAS_SELINUX
-
 #ifdef CONFIG_HAS_SELINUX
 #include <selinux/selinux.h>
 #endif
@@ -118,7 +117,7 @@ void kerndat_lsm(void)
 	if (name)
 		return;
 
-	if (access("/sys/kernel/security/apparmor", F_OK) == 0) {
+	if (access(AA_SECURITYFS_PATH, F_OK) == 0) {
 		get_label = apparmor_get_label;
 		lsmtype = LSMTYPE__APPARMOR;
 		name = "apparmor";
