@@ -10,7 +10,6 @@
 #include <stdlib.h>
 #include <dlfcn.h>
 
-#include "asm/types.h"
 #include "libnetlink.h"
 #include "cr_options.h"
 #include "imgset.h"
@@ -27,6 +26,7 @@
 #include "plugin.h"
 #include "namespaces.h"
 #include "pstree.h"
+#include "external.h"
 #include "crtools.h"
 
 #include "protobuf.h"
@@ -72,6 +72,7 @@ struct unix_sk_desc {
 	unsigned int		nr_icons;
 	unsigned int		*icons;
 	unsigned char		shutdown;
+	bool			deleted;
 
 	mode_t			mode;
 	uid_t			uid;
@@ -94,11 +95,6 @@ struct unix_sk_listen_icon {
 	struct unix_sk_listen_icon	*next;
 };
 
-struct  unix_sk_exception {
-	struct list_head unix_sk_list;
-	ino_t unix_sk_ino;
-};
-
 #define SK_HASH_SIZE		32
 
 static struct unix_sk_listen_icon *unix_listen_icons[SK_HASH_SIZE];
@@ -163,20 +159,17 @@ static int can_dump_unix_sk(const struct unix_sk_desc *sk)
 	return 1;
 }
 
-static bool unix_sk_exception_lookup_id(ino_t ino)
+static bool unix_sk_exception_lookup_id(unsigned int ino)
 {
-	bool ret = false;
-	struct unix_sk_exception *sk;
+	char id[20];
 
-	list_for_each_entry(sk, &opts.ext_unixsk_ids, unix_sk_list) {
-		if (sk->unix_sk_ino == ino) {
-			pr_debug("Found ino %u in exception unix sk list\n", (unsigned int)ino);
-			ret = true;
-			break;
-		}
+	snprintf(id, sizeof(id), "unix[%u]", ino);
+	if (external_lookup_id(id)) {
+		pr_debug("Found ino %u in exception unix sk list\n", (unsigned int)ino);
+		return true;
 	}
 
-	return ret;
+	return false;
 }
 
 static int write_unix_entry(struct unix_sk_desc *sk)
@@ -337,6 +330,11 @@ static int dump_one_unix_fd(int lfd, u32 id, const struct fd_parms *p)
 		perms->gid	= userns_gid(sk->gid);
 	}
 
+	if (sk->deleted) {
+		ue->has_deleted = true;
+		ue->deleted	= sk->deleted;
+	}
+
 	sk_encode_shutdown(ue, sk->shutdown);
 
 	if (ue->peer) {
@@ -503,7 +501,7 @@ static int unix_process_name(struct unix_sk_desc *d, const struct unix_diag_msg
 
 	if (name[0] != '\0') {
 		struct unix_diag_vfs *uv;
-		bool drop_path = false;
+		bool deleted = false;
 		char rpath[PATH_MAX];
 		struct ns_id *ns;
 		struct stat st;
@@ -554,30 +552,21 @@ static int unix_process_name(struct unix_sk_desc *d, const struct unix_diag_msg
 
 			pr_info("unix: Dropping path %s for unlinked sk %#x\n",
 				name, m->udiag_ino);
-			drop_path = true;
+			deleted = true;
 		} else if ((st.st_ino != uv->udiag_vfs_ino) ||
 			   !phys_stat_dev_match(st.st_dev, uv->udiag_vfs_dev, ns, name)) {
 			pr_info("unix: Dropping path %s for unlinked bound "
 				"sk %#x.%#x real %#x.%#x\n",
 				name, (int)st.st_dev, (int)st.st_ino,
 				(int)uv->udiag_vfs_dev, (int)uv->udiag_vfs_ino);
-			drop_path = true;
-		}
-
-		if (drop_path) {
-			/*
-			 * When a socket is bound to unlinked file, we
-			 * just drop his name, since no one will access
-			 * it via one.
-			 */
-			xfree(name);
-			len = 0;
-			name = NULL;
+			deleted = true;
 		}
 
 		d->mode = st.st_mode;
 		d->uid	= st.st_uid;
 		d->gid	= st.st_gid;
+
+		d->deleted = deleted;
 	}
 
 postprone:
@@ -705,16 +694,17 @@ static int dump_external_sockets(struct unix_sk_desc *peer)
 
 		ret = run_plugins(DUMP_UNIX_SK, sk->fd, sk->sd.ino);
 		if (ret == -ENOTSUP) {
-			if (!opts.ext_unix_sk) {
-				show_one_unix("Runaway socket", peer);
-				pr_err("External socket is used. "
-						"Consider using --" USK_EXT_PARAM " option.\n");
-				return -1;
-			}
-
 			if (unix_sk_exception_lookup_id(sk->sd.ino)) {
 				pr_debug("found exception for unix name-less external socket.\n");
 			} else {
+				/* Legacy -x|--ext-unix-sk option handling */
+				if (!opts.ext_unix_sk) {
+					show_one_unix("Runaway socket", peer);
+					pr_err("External socket is used. "
+							"Consider using --" USK_EXT_PARAM " option.\n");
+					return -1;
+				}
+
 				if (peer->type != SOCK_DGRAM) {
 					show_one_unix("Ext stream not supported", peer);
 					pr_err("Can't dump half of stream unix connection.\n");
@@ -753,7 +743,8 @@ int fix_external_unix_sockets(void)
 		FownEntry fown = FOWN_ENTRY__INIT;
 		SkOptsEntry skopts = SK_OPTS_ENTRY__INIT;
 
-		if (sk->sd.already_dumped)
+		if (sk->sd.already_dumped ||
+		    list_empty(&sk->peer_list))
 			continue;
 
 		show_one_unix("Dumping extern", sk);
@@ -962,8 +953,47 @@ static int bind_unix_sk(int sk, struct unix_sk_info *ui)
 		ret = bind(sk, (struct sockaddr *)&addr,
 				sizeof(addr.sun_family) + ui->ue->name.len);
 		if (ret < 0) {
-			pr_perror("Can't bind socket");
-			goto done;
+			if (ui->ue->has_deleted && ui->ue->deleted && errno == EADDRINUSE) {
+				char temp[PATH_MAX];
+
+				pr_info("found duplicate unix socket bound at %s\n", addr.sun_path);
+
+				ret = snprintf(temp, sizeof(temp), "%s-%s-%d", addr.sun_path, "criu-temp", getpid());
+				/* this shouldn't happen, since sun_addr is only 108 chars long */
+				if (ret < 0 || ret >= sizeof(temp)) {
+					pr_err("snprintf of %s failed?\n", addr.sun_path);
+					goto done;
+				}
+
+				ret = rename(addr.sun_path, temp);
+				if (ret < 0) {
+					pr_perror("couldn't move socket for binding");
+					goto done;
+				}
+
+				ret = bind(sk, (struct sockaddr *)&addr,
+						sizeof(addr.sun_family) + ui->ue->name.len);
+				if (ret < 0) {
+					pr_perror("Can't bind socket after move");
+					goto done;
+				}
+
+				ret = rename(temp, addr.sun_path);
+				if (ret < 0) {
+					pr_perror("couldn't move socket back");
+					goto done;
+				}
+
+				/* we've handled the deleted-ness of this
+				 * socket and we don't want to delete it later
+				 * since it's not /this/ socket.
+				 */
+				ui->ue->deleted = false;
+
+			} else {
+				pr_perror("Can't bind socket");
+				goto done;
+			}
 		}
 
 		if (*ui->name && ui->ue->file_perms) {
@@ -988,6 +1018,11 @@ static int bind_unix_sk(int sk, struct unix_sk_info *ui)
 				goto done;
 			}
 		}
+
+		if (ui->ue->deleted && unlink((char *)ui->ue->name.data) < 0) {
+			pr_perror("failed to unlink %s", ui->ue->name.data);
+			goto done;
+		}
 	}
 
 	if (ui->ue->state != TCP_LISTEN)
@@ -1039,11 +1074,7 @@ static int open_unixsk_pair_master(struct unix_sk_info *ui)
 	if (shutdown_unix_sk(sk[0], ui))
 		return -1;
 
-	tsk = socket(PF_UNIX, SOCK_DGRAM, 0);
-	if (tsk < 0) {
-		pr_perror("Can't make transport socket");
-		return -1;
-	}
+	tsk = get_service_fd(TRANSPORT_FD_OFF);
 
 	fle = file_master(&peer->d);
 	if (send_fd_to_peer(sk[1], fle, tsk)) {
@@ -1051,7 +1082,6 @@ static int open_unixsk_pair_master(struct unix_sk_info *ui)
 		return -1;
 	}
 
-	close(tsk);
 	close(sk[1]);
 
 	return sk[0];
@@ -1410,19 +1440,15 @@ static int resolve_unix_peers(void *unused)
 	return 0;
 }
 
-int unix_sk_id_add(ino_t ino)
+int unix_sk_id_add(unsigned int ino)
 {
-	struct unix_sk_exception *unix_sk;
+	char *e_str;
 
-	/* TODO: validate inode here? */
-
-	unix_sk = xmalloc(sizeof *unix_sk);
-	if (unix_sk == NULL)
+	e_str = xmalloc(20);
+	if (!e_str)
 		return -1;
-	unix_sk->unix_sk_ino = ino;
-	list_add_tail(&unix_sk->unix_sk_list, &opts.ext_unixsk_ids);
-
-	return 0;
+	snprintf(e_str, 20, "unix[%u]", ino);
+	return add_external(e_str);
 }
 
 int unix_sk_ids_parse(char *optarg)
@@ -1438,7 +1464,7 @@ int unix_sk_ids_parse(char *optarg)
 		if (*iter == ',')
 			iter++;
 		else {
-			ino_t ino = (ino_t)strtoul(iter, &iter, 10);
+			unsigned int ino = strtoul(iter, &iter, 10);
 
 			if (0 == ino) {
 				pr_err("Can't parse unix socket inode from optarg: %s\n", optarg);
